# -*- encoding: utf-8 -*-
##############################################################################
#
#    Avanzosc - Avanced Open Source Consulting
#    Copyright (C) 2011 - 2012 Avanzosc <http://www.avanzosc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv
from osv import fields
import netsvc

class pos_order_line(osv.osv):

    _inherit = 'pos.order.line' 
    
    def onchange_product_id(self, cr, uid, ids, shop_id, pricelist, product_id, qty=0, partner_id=False):
        res = super(pos_order_line, self).onchange_product_id(cr, uid, ids, pricelist, product_id, qty, partner_id)
        prodlot_id, cont, prod_list = self.get_prodlot(cr, uid, ids, shop_id, product_id) 
        if prodlot_id:
            res['value']['prodlot_id'] = prodlot_id
            res['value']['prodcont'] = cont
        res['value']['prodlot_list'] = '['+', '.join(["%s" % elem for elem in prod_list])+']'
        return res
 
    _columns = {
            'prodlot_id': fields.many2one('stock.production.lot', 'Production Lot', help="Production lot is used to put a serial number on the production"),
            'prodcont': fields.integer('Number of lots'),
            'prodlot_list': fields.char('List of Product Lots', size=64),
    }
    
    _defaults = {  
        'prodcont': lambda *a: 0,
    }
    
    def get_prodlot(self, cr, uid, ids, shop_id, product_id):
        prodlot_objs = self.pool.get('stock.production.lot')
        report_prod_objs = self.pool.get('stock.report.prodlots')
        prod_list = []
        prodlot = False
        prodlot_min_id = False
        shop = self.pool.get('sale.shop').browse(cr, uid, shop_id)
        prodlot_ids = prodlot_objs.search(cr, uid, [('product_id', '=', product_id)])
	if not shop.warehouse_id.lot_stock_id:
	    raise osv.except_osv('User error', 'The warehouse is not correctly set up !')
        for report_prod in report_prod_objs.browse(cr, uid, report_prod_objs.search(cr, uid, [('location_id', '=', shop.warehouse_id.lot_stock_id.id)])):
            if report_prod.prodlot_id:
                if (report_prod.prodlot_id.id in prodlot_ids) and (report_prod.prodlot_id.id not in prod_list):
                    prod_list.append(report_prod.prodlot_id.id)
        if len(prod_list) == 0:
            return False, len(prod_list), prod_list
        elif len(prod_list) > 1:
            prodlot_min_exp = prodlot_objs.browse(cr, uid, prod_list[0])
            prodlot_min_id = prodlot_min_exp.id
            for prodlot_id in prod_list:
                prodlot = prodlot_objs.browse(cr, uid, prodlot_id)
                if prodlot.dlc < prodlot_min_exp.dlc:
                    prodlot_min_id = prodlot_id
            return prodlot_min_id, len(prod_list), prod_list
        else:
            return prod_list[0], len(prod_list), prod_list
    
    def get_domain(self, cr, uid, ids, context=None):
        return True
pos_order_line()

class pos_order(osv.osv):

    _inherit = 'pos.order' 
    
    def create_picking(self, cr, uid, ids, context={}):
        """Create a picking for each order and validate it."""
        picking_obj = self.pool.get('stock.picking')
        partner_obj = self.pool.get('res.partner')
        address_id = False
        orders = self.browse(cr, uid, ids, context)
        for order in orders:
            if not order.last_out_picking:
                new = True
                if order.partner_id.id:
                    address_id = partner_obj.address_get(cr, uid, [order.partner_id.id], ['delivery'])['delivery']
                picking_id = picking_obj.create(cr, uid, {
                    'origin': order.name,
                    'type': 'out',
                    'state': 'draft',
                    'move_type': 'direct',
                    'note': 'POS notes ' + (order.note or ""),
                    'invoice_state': 'none',
                    'auto_picking': True,
                    'pos_order': order.id,
                    'address_id' : address_id
                    },context)
                self.write(cr, uid, [order.id], {'last_out_picking': picking_id})
            else:
                picking_id = order.last_out_picking.id
                picking_obj.write(cr, uid, [picking_id], {
                    'auto_picking': True,
                    'invoice_state': '2binvoiced',
                })
                picking = picking_obj.browse(cr, uid, [picking_id], context)[0]
                new = False

                # split the picking (if product quantity has changed):
                diff_dict = self._get_qty_differences(orders, picking)
                if diff_dict:
                    self._split_picking(cr, uid, ids, context, picking, diff_dict)

            if new:
                for line in order.lines:
                    if line.product_id.type != 'service':
                        prop_ids = self.pool.get("ir.property").search(cr, uid,
                                [('name', '=', 'property_stock_customer')])
                        val = self.pool.get("ir.property").browse(cr, uid,
                                prop_ids[0]).value
                        location_id = order.shop_id.warehouse_id.lot_stock_id.id
                        stock_dest_id = int(val.split(',')[1])
                        if line.qty < 0:
                            (location_id, stock_dest_id)= (stock_dest_id, location_id)

                        self.pool.get('stock.move').create(cr, uid, {
                            'name': 'Stock move (POS %d)' % (order.id, ),
                            'product_uom': line.product_id.uom_id.id,
                            'product_uos': line.product_id.uom_id.id,
                            'picking_id': picking_id,
                            'product_id': line.product_id.id,
                            'product_uos_qty': abs(line.qty),
                            'product_qty': abs(line.qty),
                            'tracking_id': False,
                            'prodlot_id': line.prodlot_id.id,
                            'state': 'waiting',
                            'location_id': location_id,
                            'location_dest_id': stock_dest_id,
                        })

            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking',
                    picking_id, 'button_confirm', cr)
            self.pool.get('stock.picking').force_assign(cr,
                    uid, [picking_id], context)

        return True
    
pos_order()

class stock_production_lot(osv.osv):

    _inherit = 'stock.production.lot'
   
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
       res = super(stock_production_lot, self).search(cr, uid, args, offset, limit, order, context, count)
       if context and ('prodlot_list' in context.keys()):
           list_id = []
           for id in res:
               if id not in context['prodlot_list']:
                   list_id.append(id)
           for id in list_id:
               res.remove(id)
       return res
   
stock_production_lot()

